==> openblas_0.2.12.orig.tar.gz: OpenBLAS-0.2.12/kernel/x86_64/zaxpy_microk_bulldozer-2.c <==
/***************************************************************************
Copyright (c) 2014, The OpenBLAS Project
All rights reserved.
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:
1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with the
distribution.
3. Neither the name of the OpenBLAS project nor the names of
its contributors may be used to endorse or promote products
derived from this software without specific prior written permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE OPENBLAS PROJECT OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*****************************************************************************/

#define HAVE_KERNEL_4 1
static void zaxpy_kernel_4( BLASLONG n, FLOAT *x, FLOAT *y , FLOAT *alpha) __attribute__ ((noinline));

static void zaxpy_kernel_4( BLASLONG n, FLOAT *x, FLOAT *y, FLOAT *alpha)
{


	BLASLONG register i = 0;

	__asm__  __volatile__
	(
	"vmovddup		(%4), %%xmm0		    \n\t"  // real part of alpha
	"vmovddup	       8(%4), %%xmm1		    \n\t"  // imag part of alpha

	".align 16				            \n\t"
	".L01LOOP%=:				            \n\t"

	"prefetcht0  768(%2,%0,8)                           \n\t"
	"vmovups        (%2,%0,8), %%xmm5                   \n\t" // 1 complex values from x
	"vmovups      16(%2,%0,8), %%xmm7                   \n\t" // 1 complex values from x
	"vmovups      32(%2,%0,8), %%xmm9                   \n\t" // 1 complex values from x
	"vmovups      48(%2,%0,8), %%xmm11                  \n\t" // 1 complex values from x
	"prefetcht0  768(%3,%0,8)                           \n\t"

#if !defined(CONJ)
	"vfmaddpd    (%3,%0,8), %%xmm0 , %%xmm5, %%xmm12    \n\t"
	"vpermilpd	$0x1 , %%xmm5 , %%xmm4 	    \n\t"  // exchange real and imag part
	"vmulpd         %%xmm1, %%xmm4 , %%xmm4             \n\t"

	"vfmaddpd  16(%3,%0,8), %%xmm0 , %%xmm7, %%xmm13    \n\t"
	"vpermilpd	$0x1 , %%xmm7 , %%xmm6 	    \n\t"  // exchange real and imag part
	"vmulpd         %%xmm1, %%xmm6 , %%xmm6             \n\t"

	"vfmaddpd  32(%3,%0,8), %%xmm0 , %%xmm9, %%xmm14    \n\t"
	"vpermilpd	$0x1 , %%xmm9 , %%xmm8 	    \n\t"  // exchange real and imag part
	"vmulpd         %%xmm1, %%xmm8 , %%xmm8             \n\t"

	"vfmaddpd  48(%3,%0,8), %%xmm0 , %%xmm11,%%xmm15    \n\t"
	"vpermilpd	$0x1 , %%xmm11, %%xmm10 	    \n\t"  // exchange real and imag part
	"vmulpd         %%xmm1, %%xmm10, %%xmm10            \n\t"

        "vaddsubpd      %%xmm4, %%xmm12, %%xmm12            \n\t"
        "vaddsubpd      %%xmm6, %%xmm13, %%xmm13            \n\t"
        "vaddsubpd      %%xmm8, %%xmm14, %%xmm14            \n\t"
        "vaddsubpd      %%xmm10,%%xmm15, %%xmm15            \n\t"

#else

	"vmulpd		%%xmm0, %%xmm5, %%xmm4		    \n\t" // a_r*x_r, a_r*x_i
	"vmulpd		%%xmm1, %%xmm5, %%xmm5 		    \n\t" // a_i*x_r, a_i*x_i
	"vmulpd		%%xmm0, %%xmm7, %%xmm6		    \n\t" // a_r*x_r, a_r*x_i
	"vmulpd		%%xmm1, %%xmm7, %%xmm7 		    \n\t" // a_i*x_r, a_i*x_i
	"vmulpd		%%xmm0, %%xmm9, %%xmm8		    \n\t" // a_r*x_r, a_r*x_i
	"vmulpd		%%xmm1, %%xmm9, %%xmm9 		    \n\t" // a_i*x_r, a_i*x_i
	"vmulpd		%%xmm0, %%xmm11, %%xmm10	    \n\t" // a_r*x_r, a_r*x_i
	"vmulpd		%%xmm1, %%xmm11, %%xmm11	    \n\t" // a_i*x_r, a_i*x_i

	"vpermilpd	$0x1 , %%xmm4 , %%xmm4 	    \n\t"  // exchange real and imag part
	"vaddsubpd	%%xmm4 ,%%xmm5 , %%xmm4 	    \n\t"
	"vpermilpd	$0x1 , %%xmm4 , %%xmm4 	    \n\t"  // exchange real and imag part

	"vpermilpd	$0x1 , %%xmm6 , %%xmm6 	    \n\t"  // exchange real and imag part
	"vaddsubpd	%%xmm6 ,%%xmm7 , %%xmm6 	    \n\t"
	"vpermilpd	$0x1 , %%xmm6 , %%xmm6 	    \n\t"  // exchange real and imag part

	"vpermilpd	$0x1 , %%xmm8 , %%xmm8 	    \n\t"  // exchange real and imag part
	"vaddsubpd	%%xmm8 ,%%xmm9 , %%xmm8 	    \n\t"
	"vpermilpd	$0x1 , %%xmm8 , %%xmm8 	    \n\t"  // exchange real and imag part

	"vpermilpd	$0x1 , %%xmm10, %%xmm10	    \n\t"  // exchange real and imag part
	"vaddsubpd	%%xmm10,%%xmm11, %%xmm10	    \n\t"
	"vpermilpd	$0x1 , %%xmm10, %%xmm10	    \n\t"  // exchange real and imag part

	"vaddpd	     (%3,%0,8) ,%%xmm4 , %%xmm12	    \n\t"
	"vaddpd	   16(%3,%0,8) ,%%xmm6 , %%xmm13	    \n\t"
	"vaddpd	   32(%3,%0,8) ,%%xmm8 , %%xmm14	    \n\t"
	"vaddpd	   48(%3,%0,8) ,%%xmm10, %%xmm15	    \n\t"


#endif

	"vmovups	%%xmm12,   (%3,%0,8)		    \n\t"
	"vmovups	%%xmm13, 16(%3,%0,8)		    \n\t"
	"vmovups	%%xmm14, 32(%3,%0,8)		    \n\t"
	"vmovups	%%xmm15, 48(%3,%0,8)		    \n\t"

	"addq		$8 , %0	  	 	             \n\t"
	"subq	        $4 , %1			             \n\t"		
	"jnz		.L01LOOP%=		             \n\t"

	:
        : 
          "r" (i),	// 0	
	  "r" (n),  	// 1
          "r" (x),      // 2
          "r" (y),      // 3
          "r" (alpha)   // 4
	: "cc", 
	  "%xmm0", "%xmm1",
	  "%xmm4", "%xmm5", "%xmm6", "%xmm7", 
	  "%xmm8", "%xmm9", "%xmm10", "%xmm11", 
	  "%xmm12", "%xmm13", "%xmm14", "%xmm15",
	  "memory"
	);

}

==> openblas_0.2.14.orig.tar.gz: OpenBLAS-0.2.14/kernel/x86_64/zaxpy_microk_bulldozer-2.c <==
/***************************************************************************
Copyright (c) 2014, The OpenBLAS Project
All rights reserved.
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:
1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with the
distribution.
3. Neither the name of the OpenBLAS project nor the names of
its contributors may be used to endorse or promote products
derived from this software without specific prior written permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE OPENBLAS PROJECT OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*****************************************************************************/

#define HAVE_KERNEL_4 1
static void zaxpy_kernel_4( BLASLONG n, FLOAT *x, FLOAT *y , FLOAT *alpha) __attribute__ ((noinline));

static void zaxpy_kernel_4( BLASLONG n, FLOAT *x, FLOAT *y, FLOAT *alpha)
{


	BLASLONG register i = 0;

	__asm__  __volatile__
	(
	"vmovddup		(%4), %%xmm0		    \n\t"  // real part of alpha
	"vmovddup	       8(%4), %%xmm1		    \n\t"  // imag part of alpha

	".align 16				            \n\t"
	"1:				            \n\t"

	"prefetcht0  768(%2,%0,8)                           \n\t"
	"vmovups        (%2,%0,8), %%xmm5                   \n\t" // 1 complex values from x
	"vmovups      16(%2,%0,8), %%xmm7                   \n\t" // 1 complex values from x
	"vmovups      32(%2,%0,8), %%xmm9                   \n\t" // 1 complex values from x
	"vmovups      48(%2,%0,8), %%xmm11                  \n\t" // 1 complex values from x
	"prefetcht0  768(%3,%0,8)                           \n\t"

#if !defined(CONJ)
	"vfmaddpd    (%3,%0,8), %%xmm0 , %%xmm5, %%xmm12    \n\t"
	"vpermilpd	$0x1 , %%xmm5 , %%xmm4 	    \n\t"  // exchange real and imag part
	"vmulpd         %%xmm1, %%xmm4 , %%xmm4             \n\t"

	"vfmaddpd  16(%3,%0,8), %%xmm0 , %%xmm7, %%xmm13    \n\t"
	"vpermilpd	$0x1 , %%xmm7 , %%xmm6 	    \n\t"  // exchange real and imag part
	"vmulpd         %%xmm1, %%xmm6 , %%xmm6             \n\t"

	"vfmaddpd  32(%3,%0,8), %%xmm0 , %%xmm9, %%xmm14    \n\t"
	"vpermilpd	$0x1 , %%xmm9 , %%xmm8 	    \n\t"  // exchange real and imag part
	"vmulpd         %%xmm1, %%xmm8 , %%xmm8             \n\t"

	"vfmaddpd  48(%3,%0,8), %%xmm0 , %%xmm11,%%xmm15    \n\t"
	"vpermilpd	$0x1 , %%xmm11, %%xmm10 	    \n\t"  // exchange real and imag part
	"vmulpd         %%xmm1, %%xmm10, %%xmm10            \n\t"

        "vaddsubpd      %%xmm4, %%xmm12, %%xmm12            \n\t"
        "vaddsubpd      %%xmm6, %%xmm13, %%xmm13            \n\t"
        "vaddsubpd      %%xmm8, %%xmm14, %%xmm14            \n\t"
        "vaddsubpd      %%xmm10,%%xmm15, %%xmm15            \n\t"

#else

	"vmulpd		%%xmm0, %%xmm5, %%xmm4		    \n\t" // a_r*x_r, a_r*x_i
	"vmulpd		%%xmm1, %%xmm5, %%xmm5 		    \n\t" // a_i*x_r, a_i*x_i
	"vmulpd		%%xmm0, %%xmm7, %%xmm6		    \n\t" // a_r*x_r, a_r*x_i
	"vmulpd		%%xmm1, %%xmm7, %%xmm7 		    \n\t" // a_i*x_r, a_i*x_i
	"vmulpd		%%xmm0, %%xmm9, %%xmm8		    \n\t" // a_r*x_r, a_r*x_i
	"vmulpd		%%xmm1, %%xmm9, %%xmm9 		    \n\t" // a_i*x_r, a_i*x_i
	"vmulpd		%%xmm0, %%xmm11, %%xmm10	    \n\t" // a_r*x_r, a_r*x_i
	"vmulpd		%%xmm1, %%xmm11, %%xmm11	    \n\t" // a_i*x_r, a_i*x_i

	"vpermilpd	$0x1 , %%xmm4 , %%xmm4 	    \n\t"  // exchange real and imag part
	"vaddsubpd	%%xmm4 ,%%xmm5 , %%xmm4 	    \n\t"
	"vpermilpd	$0x1 , %%xmm4 , %%xmm4 	    \n\t"  // exchange real and imag part

	"vpermilpd	$0x1 , %%xmm6 , %%xmm6 	    \n\t"  // exchange real and imag part
	"vaddsubpd	%%xmm6 ,%%xmm7 , %%xmm6 	    \n\t"
	"vpermilpd	$0x1 , %%xmm6 , %%xmm6 	    \n\t"  // exchange real and imag part

	"vpermilpd	$0x1 , %%xmm8 , %%xmm8 	    \n\t"  // exchange real and imag part
	"vaddsubpd	%%xmm8 ,%%xmm9 , %%xmm8 	    \n\t"
	"vpermilpd	$0x1 , %%xmm8 , %%xmm8 	    \n\t"  // exchange real and imag part

	"vpermilpd	$0x1 , %%xmm10, %%xmm10	    \n\t"  // exchange real and imag part
	"vaddsubpd	%%xmm10,%%xmm11, %%xmm10	    \n\t"
	"vpermilpd	$0x1 , %%xmm10, %%xmm10	    \n\t"  // exchange real and imag part

	"vaddpd	     (%3,%0,8) ,%%xmm4 , %%xmm12	    \n\t"
	"vaddpd	   16(%3,%0,8) ,%%xmm6 , %%xmm13	    \n\t"
	"vaddpd	   32(%3,%0,8) ,%%xmm8 , %%xmm14	    \n\t"
	"vaddpd	   48(%3,%0,8) ,%%xmm10, %%xmm15	    \n\t"


#endif

	"vmovups	%%xmm12,   (%3,%0,8)		    \n\t"
	"vmovups	%%xmm13, 16(%3,%0,8)		    \n\t"
	"vmovups	%%xmm14, 32(%3,%0,8)		    \n\t"
	"vmovups	%%xmm15, 48(%3,%0,8)		    \n\t"

	"addq		$8 , %0	  	 	             \n\t"
	"subq	        $4 , %1			             \n\t"		
	"jnz		1b		             \n\t"

	:
        : 
          "r" (i),	// 0	
	  "r" (n),  	// 1
          "r" (x),      // 2
          "r" (y),      // 3
          "r" (alpha)   // 4
	: "cc", 
	  "%xmm0", "%xmm1",
	  "%xmm4", "%xmm5", "%xmm6", "%xmm7", 
	  "%xmm8", "%xmm9", "%xmm10", "%xmm11", 
	  "%xmm12", "%xmm13", "%xmm14", "%xmm15",
	  "memory"
	);

}
